package at.jku.students.map;

import at.jku.ssw.Iterator;
import at.jku.ssw.list.impl.LinkedList;
import at.jku.ssw.map.HashMapUtil;
import at.jku.ssw.map.Map;
import at.jku.ssw.map.MapIterator;
import at.jku.ssw.map.Pair;
import at.jku.ssw.util.PrimeNumbers;

/**
 * An implementation of the {@linkplain Map} class based on the
 * {@linkplain Object#hashCode()} of the key objects that uses quadratic probing
 * as its collision strategy.
 */
public final class HashMapQuadraticProbing extends Map {

	protected final int tabSize;
	protected Pair[] table;
	private int size;

	public HashMapQuadraticProbing(int size) {
		this.tabSize = PrimeNumbers.calculateTabSize(size);
		table = new Pair[tabSize];
	}

	private int hash(Object val) {
		return HashMapUtil.mod(val.hashCode(), tabSize);
	}

	private static boolean isDeleted(Object o) {
		return o == Pair.DELETED_MARK;
	}

	@Override
	public boolean containsKey(Object key) {
		// TODO
		return false;
	}

	@Override
	public Object get(Object key) {
		// TODO
		return null;
	}

	@Override
	public void put(Object key, Object value) {
		// TODO
	}

	@Override
	public Object remove(Object key) {
		// TODO
		return null;
	}

	@Override
	public int size() {
		// TODO
		return -1;
	}

	@Override
	public Iterator iterator() {
		LinkedList l = new LinkedList();
		for (int i = 0; i < tabSize; i++) {
			Pair e = table[i];
			if (e != null && !isDeleted(e)) {
				l.insert(l.size(), new Pair(e.key, e.value));
			}
		}
		return new MapIterator(l);
	}
}
